# SPDX-License-Identifier: LGPL-2.1-or-later

# Copyright (c) 2019 Red Hat, Inc.
# Copyright (c) 2019 Tomáš Mráz <tmraz@fedoraproject.org>

from .cryptopolicies import UnscopedCryptoPolicy

__all__ = ['UnscopedCryptoPolicy']
